DELIMITER GO

DROP PROCEDURE IF EXISTS CreateTempTables;

GO

CREATE PROCEDURE CreateTempTables ()
BEGIN 
	CREATE TABLE ASPStateTempSessions (
		SessionId nvarchar(88) NOT NULL , 
		Created timestamp  NOT NULL DEFAULT  current_timestamp, 
		Expires timestamp  NOT NULL , 
		LockDate timestamp  NOT NULL , 
		LockDateLocal timestamp  NOT NULL , 
		LockCookie integer  NOT NULL default 0, 
		Timeout integer  NOT NULL , 
		Locked tinyint(1) NOT NULL , 
		SessionItem longblob , 
		Flags integer not null default 0,
		primary key (SessionId));

	CREATE INDEX OSIDX_ASPSESSIONS_Expire ON ASPStateTempSessions (Expires);	

	CREATE TABLE ASPStateTempApplications (
		AppId integer  NOT NULL , 
		AppName nvarchar(280) NOT NULL , 
		PRIMARY KEY (AppId) );   		
		
	-- creates index on the first 100 chars of the column to bypass the mysql index size limit
	CREATE INDEX OSIDX_ASPTEMPAPPS_App ON ASPStateTempApplications  (AppName(100));

	CREATE TABLE ASPStateTempSessionsExtVars (
		SessionId nvarchar(88) NOT NULL , 
		Variable nvarchar(100) NOT NULL , 
		Data longblob ,
		UserId integer ,
		Cookie nvarchar(88));

	CREATE INDEX OSIDX_ASPEXT_SessionId ON ASPStateTempSessionsExtVars(SessionId);
	CREATE INDEX OSIDX_ASPEXT_Variable ON ASPStateTempSessionsExtVars  (SessionId, Variable);
END;

GO

DROP PROCEDURE IF EXISTS DropTempTables;

GO

CREATE PROCEDURE DropTempTables ()
BEGIN 
	DROP TABLE IF EXISTS ASPStateTempSessions;
	DROP TABLE IF EXISTS ASPStateTempApplications;
	DROP TABLE IF EXISTS ASPStateTempSessionsExtVars;
END;

GO

call DROPTEMPTABLES()

GO

call CREATETEMPTABLES()

GO

DROP PROCEDURE IF EXISTS DeleteExpiredSessions;

GO

create PROCEDURE DeleteExpiredSessions ()
begin

	DELETE FROM ASPStateTempSessionsExtVars
	WHERE SessionId IN (SELECT SessionId FROM ASPStateTempSessions WHERE Expires < now());
					   
	DELETE FROM ASPStateTempSessions   
	WHERE  Expires  < now();
END;

GO

DROP PROCEDURE IF EXISTS DeleteExpiredSessionsIncr;

GO

CREATE PROCEDURE DeleteExpiredSessionsIncr (out deletedRowsCount integer)
begin
	set deletedRowsCount = 0;

	DELETE FROM ASPStateTempSessions   
	WHERE  Expires < now()
	LIMIT 100;
	
	SELECT row_count() into deletedRowsCount;
  
	DELETE FROM ASPStateTempSessionsExtVars
	WHERE NOT exists (SELECT * FROM ASPStateTempSessions tempSessions 
					  where ASPStateTempSessionsExtVars.sessionid=tempSessions.sessionid )
	LIMIT 100;
END;

GO

DROP PROCEDURE IF EXISTS ResetData;

GO

CREATE PROCEDURE ResetData ()
BEGIN
  CALL DROPTEMPTABLES();
  
  CALL CREATETEMPTABLES();
 
END;

GO

DROP PROCEDURE IF EXISTS TempGetAppID2;

GO

CREATE PROCEDURE TempGetAppID2 (in appName nvarchar(280), in appNamehash integer, out existingAppName nvarchar(280), out existingAppNameHash integer)
begin
	declare appNameLower nvarchar(280);

	declare continue handler for sqlstate '23000' 
		select ASPStateTempApplications.AppName into existingAppName
		from ASPStateTempApplications
		where ASPStateTempApplications.AppId = appNamehash
		limit 1;

	set appNameLower = lower(appName);
	set existingAppName = null;
	set existingAppNameHash = null;

	
	select ASPStateTempApplications.appId into existingAppNameHash
	from ASPStateTempApplications
	where ASPStateTempApplications.Appname = appNameLower;

	if existingAppNameHash is null then 
		savepoint tran1;

		select ASPStateTempApplications.appId into existingAppNameHash
		from ASPStateTempApplications
		where ASPStateTempApplications.Appname = appNameLower
		for update;

		if existingAppNameHash is null then 		
			insert into ASPStateTempApplications (AppId, AppName) values (appNameHash, appNameLower);
			set existingAppNameHash = appNameHash;
		end if;
	end if;
end;

GO	

DROP PROCEDURE IF EXISTS TEMPGETSTATEITEM;

GO

CREATE PROCEDURE TEMPGETSTATEITEM ( in id nvarchar(88),
									out item longblob,
									out locked integer,
									out lockAge integer,
									out lockCookie integer,
									out actionFlags integer)
begin
	
	select if (ASPStateTempSessions.Locked, null, SessionItem),
		   ASPStateTempSessions.Locked,
		   ASPStateTempSessions.LockCookie,
		   if(ASPStateTempSessions.Locked, timestampdiff(second, ASPStateTempSessions.LockDate, now()), 0),
		   Flags
	into item, locked, lockCookie, lockAge, actionFlags
	from ASPStateTempSessions
	where SessionId = id and now() < Expires
	for update;

    update ASPStateTempSessions
	set Expires = date_add(now(), interval ASPStateTempSessions.Timeout minute),
	    Flags = 0
	where SessionId = id and now() < Expires;

	commit;
end;

GO

DROP PROCEDURE IF EXISTS TEMPGETSTATEITEMEXCLUSIVE;

GO

CREATE PROCEDURE TEMPGETSTATEITEMEXCLUSIVE ( in id nvarchar(88),
									out item longblob,
									out locked integer,
									out lockAge integer,
									out lockCookie integer,
									out actionFlags integer)
begin

	select if (ASPStateTempSessions.Locked, null, ASPStateTempSessions.SessionItem),
		   ASPStateTempSessions.Locked,
		   if (ASPStateTempSessions.Locked, ASPStateTempSessions.LockCookie, ASPStateTempSessions.LockCookie +1),
		   if(ASPStateTempSessions.Locked, timestampdiff(second, ASPStateTempSessions.LockDate, now()), 0),
		   Flags
	into item, locked, lockCookie, lockAge, actionFlags
	from ASPStateTempSessions
	where SessionId = id and now() < Expires
	for update;

    update ASPStateTempSessions
	set Expires = date_add(now(), interval ASPStateTempSessions.Timeout minute),
		LockDate = if(ASPStateTempSessions.Locked, ASPStateTempSessions.LockDate, now()),
		LockDateLocal = if(ASPStateTempSessions.Locked, ASPStateTempSessions.LockDateLocal, sysdate()),
		LockCookie = if(ASPStateTempSessions.Locked, ASPStateTempSessions.LockCookie, ASPStateTempSessions.LockCookie + 1),
		Locked = 1,
	    Flags = 0
	where SessionId = id and now() < Expires;

	commit;
end;

GO 

DROP PROCEDURE IF EXISTS TempInsertStateItem;

GO

CREATE PROCEDURE TempInsertStateItem (in id nvarchar(88), in item longblob, in timeout integer)
begin
	declare expireDate timestamp;
	set expireDate = date_add(now(), interval timeout minute);

	INSERT INTO ASPStateTempSessions ( 
		    SessionId , 
			SessionItem , 
			Timeout , 
			Expires , 
			Locked , 
			LockDate , 
			LockDateLocal , 
			LockCookie )  
	 VALUES (id , 
			item , 
			timeout,
			expireDate,
			0 , 
			now() , 
			sysdate() , 
			1 ); 
end;

GO

DROP PROCEDURE IF EXISTS TempInsertUninitializedItem;

GO

CREATE PROCEDURE TempInsertUninitializedItem (in id nvarchar(88), in item longblob, in timeout integer)
begin
	declare now timestamp;
	declare nowLocal timestamp;
	declare expireDate timestamp;
	set expireDate = date_add(now(), interval timeout minute);

	delete from ASPStateTempSessions where SessionId = id;
	delete from ASPStateTempSessionsExtVars where SessionId = id;

	INSERT INTO  ASPStateTempSessions   
			( SessionId , 
			SessionItem , 
			Timeout , 
			Expires , 
			Locked , 
			LockDate , 
			LockDateLocal , 
			LockCookie , 
			Flags )  
	 VALUES (id , 
			item , 
			timeout , 
			expireDate , 
			0 , 
			now() , 
			sysdate() , 
			1 , 
			1 ); 
end;

GO

DROP PROCEDURE IF EXISTS TempInsertInitializedItem;

GO

CREATE PROCEDURE TempInsertInitializedItem (in id nvarchar(88), in item longblob, in timeout integer)
begin
	declare expireDate timestamp;
	set expireDate = date_add(now(), interval timeout minute);

	
	INSERT INTO  ASPStateTempSessions   
			( SessionId , 
			SessionItem , 
			Timeout , 
			Expires , 
			Locked , 
			LockDate , 
			LockDateLocal , 
			LockCookie , 
			Flags )  
	 VALUES (id , 
			item , 
			timeout , 
			expireDate , 
			1 , 
			now() , 
			sysdate() , 
			1 , 
			0 ); 
end;

GO

DROP PROCEDURE IF EXISTS TempReleaseStateItemExclusive;

GO


CREATE PROCEDURE TempReleaseStateItemExclusive (in id nvarchar(88), in lockCookie int)
begin
        
	UPDATE  ASPStateTempSessions
	SET     Expires = date_add(now(), interval ASPStateTempSessions.TIMEOUT minute),
                Locked = 0 
	WHERE  SessionId  = id
        AND LockCookie  = lockCookie; 

end;

GO 

DROP PROCEDURE IF EXISTS TempRemoveStateItem;

GO

CREATE PROCEDURE TempRemoveStateItem (in id nvarchar(88), in lockCookie integer)
begin	
	DELETE FROM ASPStateTempSessions   
	WHERE  SessionId  = id
	AND LockCookie  = lockCookie; 
end;

GO 

DROP PROCEDURE IF EXISTS TempResetTimeout;

GO

CREATE PROCEDURE TempResetTimeout (in id nvarchar(88))
begin
	
	UPDATE  ASPStateTempSessions   
	SET	Expires = date_add(now(), interval ASPStateTempSessions.TIMEOUT minute) 
	WHERE  SessionId  = id; 
end;

GO 

DROP PROCEDURE IF EXISTS TempUpdateStateItem;

GO

CREATE PROCEDURE TempUpdateStateItem (in id nvarchar(88), in item longblob, in timeout integer, in lockCookie integer)
begin
	declare expireDate timestamp;
	set expireDate = date_add(now(), interval timeout minute);
        
	UPDATE ASPStateTempSessions   
	SET Expires = expireDate,	
	    SessionItem = item,	
	    Timeout = timeout,	
	    Locked = 0 
	WHERE  SessionId = id AND LockCookie = lockCookie; 
end;

GO 

DROP PROCEDURE IF EXISTS TempCreateOrUpdateStateItem;

GO

CREATE PROCEDURE TempCreateOrUpdateStateItem (in id nvarchar(88), in item longblob, in timeout integer, in lockCookie integer)
begin
	declare expireDate timestamp;
	declare i integer;
	set expireDate =  date_add(now(), interval timeout minute);

	
	UPDATE ASPStateTempSessions
	SET Expires = expireDate, SessionItem = item, Timeout = timeout, Locked = 0
	WHERE  SessionId = id AND LockCookie = lockCookie;

	select row_count() into i;

	IF i = 0 THEN

		INSERT INTO ASPStateTempSessions(SessionId, SessionItem, timeout, Expires, Locked, LockDate, LockDateLocal, LockCookie)  
		VALUES (id, item, timeout, expireDate, 0, now(), sysdate(), 1); 
	END IF;
end;

GO 

DROP PROCEDURE IF EXISTS SMSUnlockSessions;

GO

CREATE PROCEDURE SMSUnlockSessions ()
begin
	UPDATE ASPStateTempSessions SET Locked = 0 
    WHERE SUBSTR(SessionId, 10, 3) = '000' AND Locked = 1 AND date_add(LockDate, interval 3 minute) < now(); 
end;

GO

DROP PROCEDURE IF EXISTS SMSUnlockSessionsWithSubstring;

GO

create PROCEDURE SMSUnlockSessionsWithSubstring(in position integer, in matchValue nvarchar(88))
begin
	declare matchLength integer;
	set matchLength = CHAR_LENGTH(matchValue);
	
	UPDATE ASPStateTempSessions SET Locked = 0 
    WHERE SUBSTR(SessionId, position + 1, matchLength) = matchValue AND Locked = 1 AND date_add(LockDate, interval 3 minute) < now(); 

END;

GO 